/*-----------------------------------------------------------------------------
 * Name:    	leds.c
 * Purpose: 	Low level LED functions
 * Author: 		Marko Pavlin
 * (C)2105 		s54MTB
 *-----------------------------------------------------------------------------
 *
 *----------------------------------------------------------------------------*/

#include "led.h"
#include <math.h>

TIM_HandleTypeDef    TimHandle;
GPIO_InitTypeDef gpio;
uint32_t tipka;
	
/*-----------------------------------------------------------------------------
 *      Pin_LED: Array with LED assignments
 *----------------------------------------------------------------------------*/
const LED_GPIO_PIN_ID Pin_LED[] = {  // polozaj LED na shemi
  { GPIOA, GPIO_PIN_0 },   //   LED1
  { GPIOA, GPIO_PIN_1 },
  { GPIOA, GPIO_PIN_2 },

  { GPIOA, GPIO_PIN_6 },  //   LED2
  { GPIOA, GPIO_PIN_7 },
  { GPIOB, GPIO_PIN_1 },

  { GPIOA, GPIO_PIN_3 }, //   LED3
  { GPIOA, GPIO_PIN_4 },
  { GPIOA, GPIO_PIN_5 },
	

  { GPIOA, GPIO_PIN_10 }, //   LED4
  { GPIOF, GPIO_PIN_1 },
  { GPIOF, GPIO_PIN_0 },
	
};

#define LED_CLK_ENABLE \
	__HAL_RCC_GPIOA_CLK_ENABLE(); \
	__HAL_RCC_GPIOB_CLK_ENABLE(); \
	__HAL_RCC_GPIOF_CLK_ENABLE();


#define NUM_LEDS (sizeof(Pin_LED)/sizeof(LED_GPIO_PIN_ID))

uint16_t current_count = 0;                                   // To sample the counter
static uint32_t pwm_cnt = 0;																	// SW PWM counter
static volatile uint16_t LED_pwm[NUM_LEDS];									  // LEDs intensities  (0...600)

const uint16_t intens2pwm[] = { 		// Intensity to PWM lookup table
0	,
0	,
0	,
1	,
2	,
2	,
3	,
3	,
4	,
4	,
5	,
6	,
6	,
7	,
8	,
8	,
9	,
10	,
11	,
12	,
12	,
13	,
14	,
15	,
16	,
17	,
18	,
19	,
20	,
21	,
22	,
24	,
25	,
26	,
27	,
29	,
30	,
32	,
33	,
34	,
36	,
38	,
39	,
41	,
43	,
45	,
46	,
48	,
50	,
52	,
54	,
56	,
59	,
61	,
63	,
66	,
68	,
71	,
73	,
76	,
79	,
82	,
85	,
88	,
91	,
94	,
98	,
101	,
105	,
108	,
112	,
116	,
120	,
124	,
129	,
133	,
138	,
142	,
147	,
152	,
157	,
162	,
168	,
173	,
179	,
185	,
191	,
198	,
204	,
211	,
218	,
225	,
232	,
240	,
248	,
256	,
264	,
272	,
281	,
290	,
300	,
};



void Timer3_init(void)
{

  /* Set TIM3 instance */
  TimHandle.Instance = TIM3;

  /* Initialize TIMx peripheral (1kHz) as follows:
       + Period = 80 - 1
       + Prescaler = (SystemCoreClock/80000) - 1
       + ClockDivision = 0
       + Counter direction = Up
  */
  TimHandle.Init.Period            = 80 - 1;
  TimHandle.Init.Prescaler         = (uint32_t)(SystemCoreClock / 3200000) - 1;
  TimHandle.Init.ClockDivision     = 0;
  TimHandle.Init.CounterMode       = TIM_COUNTERMODE_UP;
  TimHandle.Init.RepetitionCounter = 0;

  if (HAL_TIM_Base_Init(&TimHandle) != HAL_OK)
  {
    /* Initialization Error */
    //Error_Handler();
  }

  /*##-2- Start the TIM Base generation in interrupt mode ####################*/
  /* Start Channel1 */
  if (HAL_TIM_Base_Start_IT(&TimHandle) != HAL_OK)
  {
    /* Starting Error */
    // Error_Handler();
  }

	
}


/*-----------------------------------------------------------------------------
 *      LED_Initialize:  Initialize LEDs
 *
 * Parameters: (none)
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_Initialize (void) {
  uint32_t n;

	LED_CLK_ENABLE
	
  /* Configure pins: Push-pull Output Mode (50 MHz) with Pull-down resistors */
  for (n = 0; n < NUM_LEDS; n++) {
		gpio.Pin = Pin_LED[n].num;
		gpio.Mode = GPIO_MODE_OUTPUT_PP;
		gpio.Speed = GPIO_SPEED_HIGH;
		gpio.Pull = GPIO_PULLUP;
		HAL_GPIO_Init(Pin_LED[n].port, &gpio);
		LED_pwm[n] = 0;
  }
	
	Timer3_init();
	
}


/*-----------------------------------------------------------------------------
 *      LED_Uninitialize:  Uninitialize LEDs
 *
 * Parameters: (none)
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_Uninitialize (void) {
  uint32_t n;

  /* Configure pins: Input mode, without Pull-up/down resistors */
  for (n = 0; n < NUM_LEDS; n++) {
		HAL_GPIO_DeInit(Pin_LED[n].port, Pin_LED[n].num);
  }
}


/*-----------------------------------------------------------------------------
 *      LED_On: Turns on requested LED
 *
 * Parameters:  num - LED number
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_On (uint32_t num) {
	HAL_GPIO_WritePin(Pin_LED[num].port, Pin_LED[num].num, GPIO_PIN_SET);
}

/*-----------------------------------------------------------------------------
 *      LED_Off: Turns off requested LED
 *
 * Parameters:  num - LED number
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_Off (uint32_t num) {
	HAL_GPIO_WritePin(Pin_LED[num].port, Pin_LED[num].num, GPIO_PIN_RESET);
}

/*-----------------------------------------------------------------------------
 *      LED_Out: Write value to LEDs
 *
 * Parameters:  val - value to be displayed on LEDs
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_Out (uint32_t val) {
  uint32_t n;

  for (n = 0; n < NUM_LEDS; n++) {
    if (val & (1<<n)) {
      LED_On (n);
    } else {
      LED_Off(n);
    }
  }
}

/*-----------------------------------------------------------------------------
 *      LED_Num: Get number of available LEDs
 *
 * Parameters: (none)
 * Return:      number of available LEDs
 *----------------------------------------------------------------------------*/
uint32_t LED_Num (void) {
  return (NUM_LEDS);
}


/*-----------------------------------------------------------------------------
 *      LED_Setpwm: Sets individual LED pwm rate 
 *
 * Parameters:  num - LED number
 *              pwm - LED pwm (0-600)
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_SetPwm (uint32_t num, uint8_t pwm) {
	if (num<NUM_LEDS)
	{
		LED_pwm[num] = pwm;
	}
}

/*-----------------------------------------------------------------------------
 *      LED_SetIntens: Sets individual LED intensity
 *
 * Parameters:  num - LED number
 *              intens - LED intensity (0-100 %)
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_SetIntens (uint32_t num, uint8_t intens) {
	if (num<NUM_LEDS)
	{
		LED_pwm[num] = intens2pwm[intens];
		
	}
}



/**
  * @brief  Period elapsed callback in non blocking mode
  * @param  htim : TIM handle
  * @retval None
  */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
	uint32_t n;
		pwm_cnt++;
		for (n=0; n<NUM_LEDS; n++)
		{
			if (LED_pwm[n]<pwm_cnt) 
        LED_Off(n);
			else 
				LED_On(n);;			// Software driven PWM
		}
		
		if (pwm_cnt > 300) 
		{
			// preveri tipko ob koncu PWM periode
			
			gpio.Pin = GPIO_PIN_0;
			gpio.Mode = GPIO_MODE_INPUT;
			gpio.Speed = GPIO_SPEED_HIGH;
			gpio.Pull = GPIO_PULLDOWN;
			HAL_GPIO_Init(GPIOF, &gpio);
			
			if (HAL_GPIO_ReadPin(GPIOF, GPIO_PIN_0) == GPIO_PIN_SET) tipka++; else tipka=0;
			
			if (tipka>300) 
			{
				for (n=0; n<NUM_LEDS; n++)
          LED_Off(n);
				HAL_GPIO_WritePin(GPIOA, GPIO_PIN_9, GPIO_PIN_RESET); // disable power
				while(1);
			}

			gpio.Pin = GPIO_PIN_0;
			gpio.Mode = GPIO_MODE_OUTPUT_PP;
			gpio.Speed = GPIO_SPEED_HIGH;
			gpio.Pull = GPIO_PULLUP;
			HAL_GPIO_Init(GPIOF, &gpio);
			
			pwm_cnt = 0;
		}

}


/******************************************************************************
 * accepts hue, saturation and brightness values and outputs three 8-bit color
 * values in an array (color[])
 *
 * saturation (sat) and brightness (bright) are 8-bit values.
 *
 * hue (index) is a value between 0 and 767. hue values out of range are
 * rendered as 0.
 *
 *****************************************************************************/
void hsb2rgb(uint16_t index, uint8_t sat, uint8_t bright, uint8_t *color_r, uint8_t *color_g, uint8_t *color_b ) {
  uint8_t temp[5], n = (index >> 8) % 3;
	uint8_t x = ((((index & 255) * sat) >> 8) * bright) >> 8;
  uint8_t s = ((256 - sat) * bright) >> 8;

	temp[0] = temp[3] =              s;
  temp[1] = temp[4] =          x + s;
	temp[2] =          bright - x    ;
	*color_r  = temp[n + 2];
	*color_g = temp[n + 1];
	*color_b  = temp[n    ];
}


